package com.persagy.framework.excel.core.util;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.read.metadata.ReadSheet;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.List;

/**
 * Excel 工具类
 *
 * @author
 */
@Slf4j
public class ExcelUtils {

    private static WriteSheet initWriteSheet;
    private static ReadSheet initReadSheet;

    static {
        initWriteSheet = new WriteSheet();

        initReadSheet = new ReadSheet(1, "sheet");
        initReadSheet.setHeadRowNumber(0);
    }

    /**
     * 将列表以 Excel 响应给前端
     *
     * @param response  响应
     * @param filename  文件名
     * @param sheetName Excel sheet 名
     * @param head      Excel head 头
     * @param data      数据列表哦
     * @param <T>       泛型，保证 head 和 data 类型的一致性
     * @throws IOException 写入失败的情况
     */
    public static <T> void write(HttpServletResponse response, String filename, String sheetName,
                                 Class<T> head, List<T> data) throws IOException {
        // 输出 Excel
        EasyExcel.write(response.getOutputStream(), head)
                .autoCloseStream(false) // 不要自动关闭，交给 Servlet 自己处理
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy()) // 基于 column 长度，自动适配。最大 255 宽度
                .sheet(sheetName).doWrite(data);
        // 设置 header 和 contentType。写在最后的原因是，避免报错时，响应 contentType 已经被修改了
        response.addHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(filename, "UTF-8"));
        response.setContentType("application/vnd.ms-excel;charset=UTF-8");
    }

    public static <T> List<T> raed(MultipartFile file, Class<T> head) throws IOException {
        return EasyExcel.read(file.getInputStream(), head, null)
                .autoCloseStream(false)  // 不要自动关闭，交给 Servlet 自己处理
                .doReadAllSync();
    }

    /**
     * @param filePath 文件觉得路径
     * @return
     */
    public static List<Object> read(String filePath, Class<T> head) {
        if (!StringUtils.hasText(filePath)) {
            return null;
        }
        try (InputStream fileStream = new FileInputStream(filePath);) {
            ExcelListener<Object> excelListener = new ExcelListener<Object>();
            //异步
            EasyExcel.read(fileStream, head, excelListener).sheet(1).doRead();
            //同步
            // return EasyExcel.read(fileStream).sheet(1).doReadSync();
            return excelListener.getDatas();
        } catch (Exception e) {
            log.error("读取异常：{}", filePath);
        }
        return null;
    }
}
