package com.persagy.framework.apilog;

import cn.hutool.core.exceptions.ExceptionUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.servlet.ServletUtil;
import com.alibaba.fastjson.JSON;
import com.persagy.framework.apilog.serializer.JacksonJsonSerializer;
import com.persagy.framework.apilog.serializer.SerializerWrapper;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.AfterThrowing;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

/**
 * @description:基于AOP的日志打印
 * @author:lgy
 * @data:2021/10/28 9:54
 */
@Component
@Aspect
@Slf4j
public class RequestLogAspect {

    private static final JacksonJsonSerializer serializer = new JacksonJsonSerializer();
    private SerializeConfig serializeConfig;
    private SerializerWrapper serializerWrapper;

    @Autowired
    public void setSerialize(SerializeConfig serialize) {
        this.serializeConfig = serialize;
        this.serializerWrapper = new SerializerWrapper(serializer, serializeConfig);
    }

    //	@Pointcut("execution(* com.persagy..*.controller.*.*(..))")
    @Pointcut("@within(org.springframework.stereotype.Controller) || @within(org.springframework.web.bind.annotation.RestController)")
    public void requestServer() {
    }

    @Around("requestServer()")
    public Object doAround(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {
        if (!serializeConfig.aopLog) {
            // 若切面需要被忽略，则直接执行并返回，不做后续处理
            return proceedingJoinPoint.proceed();
        }
        long start = System.currentTimeMillis();
        RequestInfo requestInfo = new RequestInfo();
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        // 获取得到 request 对象才添加请求相关字段，否则退化为普通的切面
        if (request != null) {
            requestInfo.setIp(ServletUtil.getClientIP(request));
            requestInfo.setUrl(getFullRequestUrl(request));
            requestInfo.setHttpMethod(request.getMethod());
        }
        requestInfo.setClassMethod(String.format("%s.%s", proceedingJoinPoint.getSignature().getDeclaringTypeName(), proceedingJoinPoint.getSignature().getName()));
        // 如果配置使用全参数
        boolean af = serializeConfig == null || serializeConfig.getArgsFull();
        requestInfo.setRequestParams(serializerWrapper.argsToString(proceedingJoinPoint.getArgs(), af));
        // 目标方法执行结果
        Object result = null;
        try {
            result = proceedingJoinPoint.proceed();
        }  catch (Throwable th) {
            log.warn("Request warn : {}", JSON.toJSONString(requestInfo),th);
            throw th;
        }
        // 配置明确指定为 true，才使用全返回值
        boolean rf = serializeConfig != null && serializeConfig.getResultFull();
        requestInfo.setResult(serializerWrapper.resultToString(result, rf));
        //		requestInfo.setRequestParams(getRequestParamsByProceedingJoinPoint(proceedingJoinPoint));
        //		requestInfo.setResult(result);
        requestInfo.setTimeCost(System.currentTimeMillis() - start);
        log.info("Request Info : {}", JSON.toJSONString(requestInfo));
        return result;
    }

    @AfterThrowing(pointcut = "requestServer()", throwing = "e")
    public void doAfterThrow(JoinPoint joinPoint, Exception e) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        RequestErrorInfo requestErrorInfo = new RequestErrorInfo();
        requestErrorInfo.setIp(request.getRemoteAddr());
        requestErrorInfo.setUrl(request.getRequestURL().toString());
        requestErrorInfo.setHttpMethod(request.getMethod());
        requestErrorInfo.setClassMethod(String.format("%s.%s", joinPoint.getSignature().getDeclaringTypeName(), joinPoint.getSignature().getName()));
        requestErrorInfo.setRequestParams(getRequestParamsByJoinPoint(joinPoint));
        requestErrorInfo.setException(e);
        requestErrorInfo.setStacktrace(ExceptionUtil.stacktraceToString(e).split("\r\n\t"));
        log.error("Request Error : {}", JSON.toJSONString(requestErrorInfo));
    }

    /**
     * 获取入参
     *
     * @param proceedingJoinPoint
     * @return
     */
    private Map<String, Object> getRequestParamsByProceedingJoinPoint(ProceedingJoinPoint proceedingJoinPoint) {
        // 参数名
        String[] paramNames = ((MethodSignature) proceedingJoinPoint.getSignature()).getParameterNames();
        // 参数值
        Object[] paramValues = proceedingJoinPoint.getArgs();

        return buildRequestParam(paramNames, paramValues);
    }

    private Map<String, Object> getRequestParamsByJoinPoint(JoinPoint joinPoint) {
        // 参数名
        String[] paramNames = ((MethodSignature) joinPoint.getSignature()).getParameterNames();
        // 参数值
        Object[] paramValues = joinPoint.getArgs();

        return buildRequestParam(paramNames, paramValues);
    }

    private Map<String, Object> buildRequestParam(String[] paramNames, Object[] paramValues) {
        Map<String, Object> requestParams = new HashMap<>();
        for (int i = 0; i < paramNames.length; i++) {
            Object value = paramValues[i];
            // 如果是文件对象
            if (value instanceof MultipartFile) {
                MultipartFile file = (MultipartFile) value;
                value = file.getOriginalFilename(); // 获取文件名
            } else if (value instanceof ServletRequest || value instanceof ServletResponse) {
                continue;
            }
            requestParams.put(paramNames[i], value);
        }

        return requestParams;
    }

    @Data
    public class RequestInfo {
        private String ip;
        private String url;
        private String httpMethod;
        private String classMethod;
        private Object requestParams;
        private Object result;
        private Long timeCost;
    }

    @Data
    public class RequestErrorInfo {
        private String ip;
        private String url;
        private String httpMethod;
        private String classMethod;
        private Object requestParams;
        private transient Exception exception;
        private String[] stacktrace;
    }


    @Data
    @Component
    @AllArgsConstructor
    @NoArgsConstructor
    public static class SerializeConfig {
        @Value("${api.access.log.aop.enable:true}")
        private volatile Boolean aopLog;
        @Value("${api.access.log.serialize.args-full:true}")
        private volatile Boolean argsFull;
        @Value("${api.access.log.serialize.args-max-length:2048}")
        private volatile Integer argsMaxLength;
        @Value("${api.access.log.serialize.result-full:false}")
        protected volatile Boolean resultFull;
        @Value("${api.access.log.serialize.result-max-length:2048}")
        private volatile Integer resultMaxLength;
    }

    /**
     * 获取完整的请求路径, 包含 queryString
     *
     * @param request 当前请求
     * @return 完整的请求路径
     */
    public static String getFullRequestUrl(HttpServletRequest request) {
        if (request == null) {
            return "";
        }
        if (StrUtil.isBlank(request.getQueryString())) {
            return request.getRequestURL().toString();
        }
        return request.getRequestURL().append('?').append(request.getQueryString()).toString();
    }
}